'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "lib"
directory.
'''

import data
from classes import *
from functions import *
from locals import *

import euclid
import random

import sys
import math
import os
import pygame
from pygame.locals import *
from pygame import draw

def main():
    """Cellular game
    
    Game for Pyweek 4: "The only way is up"    
    """
    #print data.load('version.txt').read()
    
    #Check if psyco is present
    try:
    	import psyco
    	psyco.full()
    except ImportError:
    	print "Psyco not present.  Consider installing it for a speedboost."
    
    #Initialize and prepare screen
    pygame.init()
    pygame.display.set_caption('Cellular')
    #icon = pygame.transform.scale(Alien.images[0], (32, 32))
    #pygame.display.set_icon(icon) 
    
    #winstyle = 0  # |FULLSCREEN
    #bestdepth = pygame.display.mode_ok(SCREENRECT.size, winstyle, 32)
    #screen = pygame.display.set_mode(SCREENRECT.size, winstyle, bestdepth)
    screen = pygame.display.set_mode((SCREEN_WIDTH, SCREEN_HEIGHT), 0, BPP)
    menu(screen)
     
    sys.exit(1)

def menu(screen):
    background, background_rect = data.load_image("menu-noscores.png")
    screen.blit(background,(0,0))
    pygame.display.flip()
    
    choice = 0
    rect = Rect(40, 289, 150, 25)
    
    back = background
    while True:
        draw.rect(back, pygame.Color("Red"), rect, 2)
        #pygame.mouse.get_pos()
        
        for e in pygame.event.get():
            if ((e.type == QUIT) or
                    (e.type is KEYDOWN and e.key == K_F4
                        and (e.mod&(KMOD_LALT|KMOD_RALT)) != 0
                        and (e.mod&(KMOD_LCTRL|KMOD_RCTRL)) == 0) or
                    (e.type is KEYDOWN and e.key == K_c
                        and (e.mod&(KMOD_LALT|KMOD_RALT)) == 0
                        and (e.mod&(KMOD_LCTRL|KMOD_RCTRL)) != 0) or
                    (e.type is KEYDOWN and e.key == K_ESCAPE)):   
                sys.exit(0)
            elif (e.type == KEYDOWN and e.key == K_BACKSPACE):
                pygame.image.save(screen, 'Cellular.bmp')
            elif e.type == KEYDOWN:
                if ((e.key == K_RIGHT) or (e.key == K_DOWN)):
                    if choice < 3:
                        draw.rect(background, pygame.Color("Black"), rect, 2)
                        rect.top += 33
                        choice += 1
                	#self.MoveDown(screen)
                elif ((e.key == K_LEFT) or (e.key == K_UP)):
                    if choice > 0:
                        draw.rect(background, pygame.Color("Black"), rect, 2)
                        rect.top -= 33
                        choice -= 1
                	#self.MoveUp(screen)
                elif ((e.key == K_SPACE) or (e.key == K_RETURN)):
                    do_choice(choice,screen)
        screen.blit(background,(0,0))
        #pygame.display.update(dirty)
        pygame.display.flip()

def do_choice(choice,screen):
    if choice == 0:
        play(screen)
    elif choice == 1:
        display_instructions(screen)
    elif choice == 2:
        pass
    elif choice == 3:
        print "Thank you for playing Cellular"
        pygame.display.quit()
        sys.exit(1)    

def display_instructions(screen):
    background, background_rect = data.load_image("instructions.png")
    screen.blit(background,(0,0))
    pygame.display.flip()

    done = False
    while not done:
        for event in pygame.event.get():
	        if event.type == pygame.QUIT: 
		        sys.exit()
	        elif event.type == KEYDOWN:
		        done = True
    
def play(screen):
    background = pygame.Surface(screen.get_size())
    
    b = ManagedBackground(background,screen)
    pygame.display.flip()
    
    #Create Groups and Sprites        
    all = ImagelessGroup()
    player = Amoeba()     
    #player = SpriteBlock()
    all.add(player)
    
    vesticles = ImagelessGroup()
    all.add(vesticles)
        
    enemies = ImagelessGroup()            
    all.add(enemies)
    
    #Other Variables    
    random.seed()
    clock = pygame.time.Clock()
    
    #Main game loop
    dty = None
    zk = 0
    pause = False
    zoomlevel = 1   
    done = False
    while not done: 
    
        #Handle player input 
        while True: #Do-while loop for pause *cough, cough*
            for e in pygame.event.get():    
                #Full Screen = Alt+Enter
                if (e.type is KEYDOWN and e.key == K_RETURN
                        and (e.mod&(KMOD_LALT|KMOD_RALT)) != 0
                        and (e.mod&(KMOD_LCTRL|KMOD_RCTRL)) == 0): 
                    screen = toggle_fullscreen()
                #Take a Screenshot
                elif (e.type == KEYDOWN and e.key == K_BACKSPACE):
                    pygame.image.save(screen, 'Cellular.bmp')
                #Respawn player if dead
                elif (e.type == KEYDOWN and e.key == K_RETURN):
                    if not player.is_alive():
                        player = Amoeba(zoomlevel=zoomlevel)
                        all.add(player)
                        
                #Pause the Game
                elif (e.type == KEYDOWN and e.key == K_p):
                    pause = not pause 
                    if pause:
                        print "Game Paused, press 'P' to unpause"
                    else:
                        print "Game Unpaused"                   
                #Return to Game Menu: Escape
                elif (e.type == KEYUP and e.key == K_ESCAPE):
                    done = True
                    pause = False
                    break
                #Quit the Game: X Button, Alt-F4, or Ctrl-C
                elif ((e.type == QUIT) or
                    (e.type is KEYDOWN and e.key == K_F4
                        and (e.mod&(KMOD_LALT|KMOD_RALT)) != 0
                        and (e.mod&(KMOD_LCTRL|KMOD_RCTRL)) == 0) or
                    (e.type is KEYDOWN and e.key == K_c
                        and (e.mod&(KMOD_LALT|KMOD_RALT)) == 0
                        and (e.mod&(KMOD_LCTRL|KMOD_RCTRL)) != 0)):   
                    sys.exit(0)
                #Create Vesticle
                elif ((e.type == KEYDOWN and e.key == K_b) or 
                      (e.type == MOUSEBUTTONDOWN and e.button == 3) 
                       and player.is_alive()):               
                    vesticle = create_vesticle(player, enemies, zoomlevel)
                    if vesticle:
                        vesticles.add(vesticle)
                        all.add(vesticle)  
                #Change this to a held down key response?
                #Create Pseudopod
                elif ((e.type == KEYDOWN and e.key == K_SPACE) or 
                      (e.type == MOUSEBUTTONDOWN and e.button == 1)
                       and player.is_alive()):
                    pr = player.get_radius()
                    md = 3 * pr/zoomlevel
                    minsize = pr/3
                    maxsize = pr/1.25  
                    enemy = find_closest(player,enemies,minsize,maxsize,md)
                    if enemy:
                        pseudo = Pseudopod(player,enemy,md)
                        all.add(pseudo) 
                    
                #Mouse Scroll Up    
                if e.type == MOUSEBUTTONDOWN and e.button == 4:
                    zk -= 2
                #Mouse Scroll Down
                if e.type == MOUSEBUTTONDOWN and e.button == 5: 
                    zk += 1
            #We can pause the game if the mouse leaves the screen
            #if not pause: 
            #    print "Game Paused, press 'P' to unpause"
            #    pause = not pygame.mouse.get_focused()
            if not pause:
                break
              
        #Clear the background over each sprites current location, erasing it.        
        all.clear(screen, background) 
    
        #Move/Updates the sprites
        all.update()
        player.move(pygame.mouse.get_pos(),screen.get_rect())        
        
        #Randomly add an enemy if player is alive
        if len(enemies) < MAX_ENEMIES and player.is_alive():
            if not int(random.random() * RANDOM_ENEMY):
                e = add_enemy(b.get_level(),zoomlevel)
                #e = Simplecell(zoomlevel)
                enemies.add(e)
                all.add(e)
        
        #Random chance of splitter        
        if ((player.get_radius()/zoomlevel > MAX_RADIUS*1.5) and 
                len(enemies) < MAX_ENEMIES) and player.is_alive():
            if not int(random.random() * RANDOM_SPLITTER):
                e = Splitter(zoomlevel)
                enemies.add(e)
                all.add(e)          
        
        #Query Pressed Keys
        keystate = pygame.key.get_pressed()            
                
        #Handle Zoom Level        
        zk += keystate[K_x] - keystate[K_z]
        if zk: zoomlevel, dty = handle_zoom(zoomlevel,zk,player,all,b)
        #Force a complete zoomdown
        if (player.get_radius()/zoomlevel == START_RADIUS or
           player.get_radius()/zoomlevel < START_RADIUS * 2 
           or zk < 0):
            zk = 0
        
        #pr = keystate[K_a] - keystate[K_s]
        #if pr: player.set_radius(player.get_radius()+(pr*zoomlevel))                
                                                          
        #Detect collisions 
        #Enemies bouncing against each other
        """
        for e1, elist in pygame.sprite.groupcollide(enemies, enemies, 0, 0).iteritems(): 
            if e1.hit == True:
                break
            for e2 in elist:                   
                if e1 == e2:
                    break
                elif e2.hit == True:
                    break               
                else:
                    Fa, Fb = vector_collision(e1,e2)  
                    e1.velocity = Fa
                    e2.velocity = Fb
                    #e2.hit = True
            #e1.hit = True
            #e1.touched = True
        """        

        #Player/Enemy Collision
        for enemy in pygame.sprite.spritecollide(player, enemies, 0):
            if not player.alive:
                break
            enemy.set_touched(True)
            if not enemy.is_hit():
                if type(enemy)==Splitter and enemy.get_radius() < player.get_radius():
                        #split(a)                        
                        #Fa, Fb = vector_collision(player,enemy)
                        #player.set_velocity(Fa)
                        player.set_radius(player.get_radius() / 2)
                        cl = player.copy()
                        c = Cell(cl.get_position(),cl.get_radius(),cl.get_velocity(),cl.get_zoomlevel(),pygame.Color("Grey"))#cl.get_color())
                            
                        c.set_velocity(-c.get_velocity())                        
                        Fa, Fc = vector_collision(player,c)
                        player.set_velocity(Fa)
                        c.set_velocity(Fc)
                        c.set_hit(True)
                        enemies.add(c)
                        all.add(c) 
                        enemy.kill()

                else: determine_outcome(player,enemy)
        
        #Vesticle/Enemy Collision
        for v in vesticles:
            for enemy in pygame.sprite.spritecollide(v, enemies, 0):
                Fa, Fb = vector_collision(v,enemy)
                v.set_velocity(Fa)
                enemy.set_velocity(Fb)
                enemy.set_hit(True)
                #v.kill()
                
        #We need to create the vesticle outside the radius of the player
        #For this to be feasable
        #for v in pygame.sprite.spritecollide(player, vesticles, 0):
        #    player.consume(v)
          
        #If an enemy wasn't hit this round, reset it's hit value
        for enemy in enemies:
            if enemy.is_touched() == False:
                enemy.set_hit(False)
            enemy.set_touched(False)
                
        #Append the sprite's new location to my dirty_rects list.
        dirty = all.draw(screen)
        if dty: #Is the background dirty?
            for d in dty:
                dirty.append(d)
            dty = None
        
        #Call display.update(dirty_rects)
        pygame.display.update(dirty) 
        
        #Game Delay
        clock.tick(FPS) 
    #End while
    print "Final Score:",player.get_radius()  
#End play
