#sprite_classes

import pygame
from pygame import draw
import random as R
import euclid
import math

#if not pygame.font: print 'Warning, fonts disabled'
#if not pygame.mixer: print 'Warning, sound disabled'

import data
from locals import *

class ManagedBackground:
    images = ['black.png','petri dish.png', 'ground.png', 'street.png', 
              'BlueMarble_2005_SAm_09_1024.jpg', 'stars.png','black.png']
     
    #offset = (initial zoom size of image, last zoomsize of image)
    #Anything under 1 for initial zoom is slow, it's much more recommended to
    #add an additional image of the zoomed screenshot
    offsets = [(1,50),(.7,50),(1,50),(1,50),(2,50),(1,50),(1,50),
                     (1,50)] 
    zoomlevel = 1           
    def __init__(self, background, screen, start_image=1):
        self.background = background
        self.brect = background.get_rect()
        self.screen = screen
      
        if start_image < 0 or start_image >= len(self.offsets):
            start_image = 0
            
        self.current = start_image
        self.image, self.rect = data.load_image(self.images[self.current])
        self.set_next()
        self.scale()

    def get_level(self):
        return self.current

    def get_center(x,y,width,height):
        return pygame.Rect(x,y,width,height)

    def set_next(self):
        self.next = self.get_next(self.current)
        self.nimage, self.nrect = data.load_image(self.images[self.next])
        self.ncenter = self.background_center(self.nrect)        
        if self.offsets[self.next][0] != 1:
            self.nscale()

    def get_next(self, current):
        if current+1 < len(self.images):
            return current+1            
        return 0

    def background_center(self,rect):
        x = rect.centerx-self.brect.width/2
        y = rect.centery-self.brect.height/2  
        width = self.brect.width
        height = self.brect.height
        return pygame.Rect(x,y,width,height)

    def increment_image(self):
        self.zoomlevel = 1
        #Speed up copy here if we have sane settings

        if self.offsets[self.next][0] == 1:
            self.image = self.nimage
            self.rect = self.nrect
        else:
            self.image, self.rect = data.load_image(self.images[self.next])    
        
        self.current = self.next
        self.set_next()  

    def background_zoom(self, zk):
        #if not (zk == 1 or zk == -1):
        #    raise "Error, need adjustment, not zoomlevel",zk
        self.zoomlevel += (self.zoomlevel * .05) * (zk)
        
        last_offset = self.offsets[self.current][1]
        if self.zoomlevel >= last_offset:
            self.increment_image()         
        return self.scale()      
    
    def scale(self):
        offset = float(self.zoomlevel)*(self.offsets[self.current][0])
        zoomx = int(self.rect.width/offset)
        zoomy = int(self.rect.height/offset) 
           
        scaled = pygame.transform.scale(self.image, (zoomx,zoomy))
        scaled = scaled.convert()                                        
        sr = scaled.get_rect()
        
        center = self.background_center(sr) 
        dirty = []
        if sr.size < self.brect.size: 
            dirty.append(self.background.fill(pygame.Color("Black"))              )
            dirty.append(self.background.blit(self.nimage, (0,0), self.ncenter))
        
        dirty.append(self.background.blit(scaled, (0,0), center))
        self.screen.blit(self.background, (0,0))
            
        return dirty

    def nscale(self):
        offset = self.offsets[self.next][0]
        zoomx = int(self.nrect.width/offset)
        zoomy = int(self.nrect.height/offset) 
            
        self.nimage = pygame.transform.scale(self.nimage, (zoomx,zoomy))
        self.nimage = self.nimage.convert()                                        
        self.nrect = self.nimage.get_rect()
        self.ncenter = self.background_center(self.nrect)

def coin_flip():
    return R.randint(0,1) == 0

class SpriteBlock(pygame.sprite.Sprite):
    touched = False
    hit = False
    def __init__(self,
            image=None, zoomlevel=1, angle=0):       
        pygame.sprite.Sprite.__init__(self)
        
        if not image:
            raise "Image cannot be None:",image
        
        self.i, self.irect = data.load_image_colorkey(image, -1)
        self.rect = self.irect
        self.zoomlevel = zoomlevel
        self.irect.height *= self.zoomlevel
        self.irect.width *= self.zoomlevel
                 
        p = euclid.Point2(0,0)                      
        if coin_flip():
            if coin_flip():
                p.x = -self.rect.height/2/zoomlevel
            else:
                p.x = SCREEN_WIDTH + self.rect.height/2/zoomlevel
            p.y = R.randint(0,SCREEN_HEIGHT)
        else:
            if coin_flip():
                p.y = -self.rect.width/2/zoomlevel
            else:
                p.y = SCREEN_HEIGHT + self.rect.width/2/zoomlevel
            p.x = R.randint(0, SCREEN_WIDTH) 
        
        v = euclid.Vector2(
            float(R.randint(0,SCREEN_WIDTH) - p.x) / SCREEN_WIDTH,
            float(R.randint(0,SCREEN_HEIGHT) - p.y) / SCREEN_HEIGHT)
        
        self.velocity = v
        self.angle = angle        
        
        self.image, self.rect = self.set_angle(self.i)
            
        self.position = p
        self.rect.center = (self.position.x,self.position.y)        


    def __copy__(self):
        return SpriteBlock(self.image, self.position.copy(), 
                self.velocity.copy(), self.zoomlevel, self.angle)
    copy = __copy__

    def get_radius(self):
        return self.get_size()
        
    def set_radius(self, radius):
        self.rect.height = radius*2
        self.rect.width = radius*2
        self.image = pygame.transform.scale(self.i, (self.rect.width,self.rect.height)) 
        self.rect = self.image.get_rect()
        self.rect.center = (self.position.x,self.position.y)

    def is_hit(self):
        return self.hit
    
    def is_touched(self):
        return self.touched
        
    def set_hit(self, hit):
        self.hit = hit
        
    def set_touched(self, touched):
        self.touched = touched

    def set_angle(self, i):
        diff = self.angle + (math.atan2(-self.velocity.x,-self.velocity.y)/ math.pi * 180.0)
        image = pygame.transform.rotate(i, diff)
        rect = image.get_rect() 
        self.angle -= diff
        return image,rect
            

    def update(self): 
        self.position += self.velocity
        self.rect.center = (self.position.x,self.position.y)

        if (self.position.x+self.rect.width/2/self.zoomlevel < 0 and 
                self.velocity.x < 0):
            self.kill()
        if (self.position.x-self.rect.width/2/self.zoomlevel > SCREEN_WIDTH and 
                self.velocity.x > 0):
            self.kill()
        if (self.position.y+self.rect.height/2/self.zoomlevel < 0 and 
                self.velocity.y < 0):
            self.kill()
        if (self.position.y-self.rect.height/2/self.zoomlevel > SCREEN_HEIGHT and 
                self.velocity.y > 0):
            self.kill() 

    def move_rel(self, pos):
        """Relative movement based on pos."""
        self.position += pos

    def is_alive(self):
        """Used to test if an object is still alive."""
        return self.alive
        
    def kill(self):
        pygame.sprite.Sprite.kill(self)
        self.alive = False

    def get_zoomlevel(self):
        return self.zoomlevel

    def get_position(self):
        return self.position

    def get_color(self):
        return pygame.color.Color("White")
        
    def get_velocity(self):
        return self.velocity

    def get_size(self):
        """Return the size of an object."""
        return ((self.rect.width + self.rect.height) / 4) * self.zoomlevel

    def get_area(self):
        return self.irect.width * self.irect.height

    def set_color(self, color):
        pass#self.color = color
        
    def set_zoomlevel(self, zoomlevel):
        """Adjust the zoomlevel of an object
        
        The zoom level determines the size of an object on the screen
        relative to it's size."""
        self.zoomlevel = zoomlevel

        self.rect.width = int(self.irect.width/self.zoomlevel)
        self.rect.height = int(self.irect.height/self.zoomlevel)
        
        #self.image, x = self.set_angle(self.i)
        image = pygame.transform.rotate(self.i, -self.angle)
        self.image = pygame.transform.scale(image, (self.rect.width,self.rect.height))
        self.rect = self.image.get_rect()
        self.rect.center = (self.position.x,self.position.y)
        
        if self.get_size() < MINR:
            self.kill() 

    def set_position(self, position):
        """Set the position of a block"""        
        self.position = position
        self.rect.center = (self.position.x,self.position.y)

    def set_velocity(self, velocity):
        self.velocity = velocity
        
        self.image, x = self.set_angle(self.image)
        #self.image = pygame.transform.scale(self.image, (self.rect.width,self.rect.height))
        self.rect.center = (self.position.x,self.position.y)        

    def consume(self, block): 
        """Consume another block, increasing the current block's size.
        
        The difference in the size of the blocks is equal to the size of the
        increase divided by a constant the represents energy needed for the
        consumption.  The consummed block is killed after it is consummed."""
        diff = ((self.get_radius() - (self.get_radius() - block.get_radius())) / 
            CONSUME_ENERGY)
        self.rect.width += diff / self.zoomlevel
        self.rect.height += diff / self.zoomlevel
        
        self.image, x = self.set_angle(self.i)
        self.image = pygame.transform.scale(self.image, (self.rect.width,self.rect.height))
        self.image = pygame.transform.rotate(self.image, -self.angle)
        self.rect = self.image.get_rect()
        
        self.rect.center = (self.position.x,self.position.y)
        block.kill()
        return diff 
    
